/*
delbt.c Version 0.1.0  Btree delete random records
from the test database
Copyright (C) 2003  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/* Usage: delbt filename */

#include <bthash.h>

#define MAXLMT 99999

void putstx(char *pgm)
   {
   fprintf(stderr,"Usage:   %s filename "
      "#records\n", pgm);
   fprintf(stderr,"Example: %s test.bt 100\n", pgm);
   fprintf(stderr,"#records is 1 - %d\n",
      MAXLMT);
   exit(1);
   } /* putstx */

int main(int argc, char **argv)
   {
   int opnerr;
   int i;
   int seq;
   int rslt;
   int lmt;
   int blocks;
   int count;
   int leaf;
   int nonleaf;
   int active;
   int deletes;
   int indxrcd;
   int leafrcd;
   int totdel;
   int maxdel;
   int delerr;
   int position;
   btreefmt *bthdr;
   unsigned char *dellst;
   unsigned char *lstptr;
   unsigned char *rndSeed;
   unsigned char *p,*q,*r;
   unsigned char buf[4096*16];
   char dbname[256];
   if (argc != 3) putstx(*argv);
   strcpy(dbname,*(argv+1));
   lmt = (int) atoi(*(argv+2));
   if (lmt < 1 || lmt > MAXLMT) putstx(*argv);

   rndSeed = (unsigned char *) rndinit();
   if (rndSeed == NULL)
      {
      fprintf(stderr,"delbt: rndinit error\n");
      exit(1);
      } /* rndinit err */

   bthdr = (btreefmt *) btopen(dbname,UPDT,&opnerr);
   if (bthdr == NULL)
      {
      fprintf(stderr,"delbt: open "
	 "error %d\n", opnerr);
      free(rndSeed);
      return(0);
      } /* open error */
   /* process starts here */

   btdbsz(bthdr,&blocks,&count);
   if (bthdr->reason)
      {
      fprintf(stderr,"delbt: btdbsz error "
         "reason %d\n",
         bthdr->reason);
      exit(1);
      } /* btdbsz err */
   dellst = (unsigned char *)
      malloc((count * bthdr->keylen) + 16);
   if (dellst == NULL)
      {
      fprintf(stderr,"delbt: out of memory\n");
      exit(1);
      } /* out of mem err */
   p = (unsigned char *) buf;
   q = (unsigned char *) p + bthdr->rcdlen;
   while (p < q) *p++ = '\0';
   lstptr = (unsigned char *) dellst;
   rslt = totdel = 0;
   position = SWPFRST;
   while (1)
      {
      rslt = btsweep(bthdr,position,NULL);
      if (rslt == NOTFOUND) break;
      if (rslt)
         {
         fprintf(stderr,"delbt: btsweep "
            "error %d reason %d\n",
            rslt, bthdr->reason);
         free(rndSeed);
         btclose(bthdr);
	 exit(1);
         } /* btsweep err */
      p = (unsigned char *) bthdr->key;
      q = (unsigned char *) p + bthdr->keylen;
      r = (unsigned char *) lstptr;
      while (p < q) *r++ = *p++;
      totdel++;
      lstptr += bthdr->keylen;
      position = SWPNEXT;
      } /* sweep loop */

   if (!totdel)
      {
      printf("delbt: empty database\n");
      free(rndSeed);
      btclose(bthdr);
      exit(1);
      } /* empty file */

   maxdel = totdel;
   seq = delerr = 0;
   if (lmt < totdel) i = lmt;
   else i = totdel;
   while (i--)
      {
      lstptr = (unsigned char *) dellst
         + (rndnum(maxdel,rndSeed) * bthdr->keylen);
      rslt = btdlet(bthdr,ROOT,lstptr);
      if (rslt == 1) /* not found */
         delerr++;
      else if (rslt)
         {
         fprintf(stderr,"delbt: delete "
            "error %d reason %d\n",
            rslt, bthdr->reason);
         free(rndSeed);
         btclose(bthdr);
         exit(1);
         } /* delete err */
      printf("%7d.  ", ++seq);
      /* print key */
      p = (unsigned char *) lstptr;
      q = (unsigned char *) p + bthdr->keylen;
      while (p < q) putchar(*p++);
      putchar('\n');
      /* shift left */
      p = (unsigned char *) lstptr + bthdr->keylen;
      q = (unsigned char *) dellst
         + (totdel * bthdr->keylen);
      r = (unsigned char *) lstptr;
      while (p < q) *r++ = *p++;
      maxdel--;
      } /* delete loop */

   btdlets(bthdr,&blocks,&count,
      &leaf,&nonleaf,&active,&deletes,
      &indxrcd,&leafrcd);
   printf("delbt: blocks %d count %d "
      "leaf %d index %d\n",
      blocks, count, leaf, nonleaf);
   printf("delbt: active %d deletes %d "
      "index rcds %d leaf rcds %d\n",
      active, deletes, indxrcd, leafrcd);
   printf("delbt: active records %d depth %d\n",
      count - deletes, btdepth(bthdr));

   /**********************************************************/
   btclose(bthdr);
   return(0);
   } /* main */
